@@ -16,15 +16,16 @@ DOMAIN_NAME=${DOMAIN_NAME:-federated_domain}
 PROJECT_NAME=${PROJECT_NAME:-federated_project}
 GROUP_NAME=${GROUP_NAME:-federated_users}
 
-# TODO(rodrigods): remove/update the settings based at testshib
-IDP_ID=${IDP_ID:-testshib}
-IDP_USERNAME=${IDP_USERNAME:-myself}
-IDP_PASSWORD=${IDP_PASSWORD:-myself}
-IDP_REMOTE_ID=${IDP_REMOTE_ID:-https://idp.testshib.org/idp/shibboleth}
-IDP_ECP_URL=${IDP_ECP_URL:-https://idp.testshib.org/idp/profile/SAML2/SOAP/ECP}
-IDP_METADATA_URL=${IDP_METADATA_URL:-http://www.testshib.org/metadata/testshib-providers.xml}
+IDP_ID=${IDP_ID:-samltest}
+IDP_USERNAME=${IDP_USERNAME:-morty}
+IDP_PASSWORD=${IDP_PASSWORD:-panic}
+IDP_REMOTE_ID=${IDP_REMOTE_ID:-https://samltest.id/saml/idp}
+IDP_ECP_URL=${IDP_ECP_URL:-https://samltest.id/idp/profile/SAML2/SOAP/ECP}
+IDP_METADATA_URL=${IDP_METADATA_URL:-https://samltest.id/saml/idp}
 
-MAPPING_REMOTE_TYPE=${MAPPING_REMOTE_TYPE:-eppn}
+KEYSTONE_IDP_METADATA_URL=${KEYSTONE_IDP_METADATA_URL:-"http://$HOST_IP/identity/v3/OS-FEDERATION/saml2/metadata"}
+
+MAPPING_REMOTE_TYPE=${MAPPING_REMOTE_TYPE:-uid}
 MAPPING_USER_NAME=${MAPPING_USER_NAME:-"{0}"}
 
 PROTOCOL_ID=${PROTOCOL_ID:-mapped}
@@ -58,23 +59,68 @@ function configure_apache {
     restart_apache_server
 }
 
+function configure_shibboleth {
+    # Copy a templated /etc/shibboleth/shibboleth2.xml file...
+    sudo cp $FEDERATION_FILES/shibboleth2.xml $SHIBBOLETH_XML
+    # ... and replace the %HOST_IP%, %IDP_REMOTE_ID%,and %IDP_METADATA_URL% placeholders
+    sudo sed -i -e "
+        s|%HOST_IP%|$HOST_IP|g;
+        s|%IDP_METADATA_URL%|$IDP_METADATA_URL|g;
+        s|%KEYSTONE_METADATA_URL%|$KEYSTONE_IDP_METADATA_URL|g;
+        " $SHIBBOLETH_XML
+
+    sudo cp "$FEDERATION_FILES/attribute-map.xml" $ATTRIBUTE_MAP
+
+    restart_service shibd
+}
+
 function install_federation {
     if is_ubuntu; then
-        install_package libapache2-mod-shib2
+        install_package libapache2-mod-shib2 xmlsec1
 
         # Create a new keypair for Shibboleth
         sudo shib-keygen -f
 
         # Enable the Shibboleth module for Apache
-        sudo a2enmod shib2
-    else
+        sudo a2enmod shib
+    elif is_fedora; then
         # NOTE(knikolla): For CentOS/RHEL, installing shibboleth is tricky
         # It requires adding a separate repo not officially supported
-        echo "Skipping installation of shibboleth for non ubuntu host"
+
+        # Add Shibboleth repository with curl
+        curl https://download.opensuse.org/repositories/security://shibboleth/CentOS_7/security:shibboleth.repo \
+        | sudo tee /etc/yum.repos.d/shibboleth.repo >/dev/null
+
+        # Install Shibboleth
+        install_package shibboleth xmlsec1-openssl
+
+        # Create a new keypair for Shibboleth
+        sudo /etc/shibboleth/keygen.sh -f -o /etc/shibboleth
+
+        # Start Shibboleth module
+        start_service shibd
+    elif is_suse; then
+        # Install Shibboleth
+        install_package shibboleth-sp
+        # Install xmlsec dependency needed only for opensuse
+        install_package libxmlsec1-openssl1
+
+        # Create a new keypair for Shibboleth
+        sudo /etc/shibboleth/keygen.sh -f -o /etc/shibboleth
+
+        # Start Shibboleth module
+        start_service shibd
+    else
+        echo "Skipping installation of shibboleth for non ubuntu nor fedora nor suse host"
     fi
+
+    pip_install pysaml2
+
+    # xmlsec1 needed for k2k
+    install_package xmlsec1
 }
 
-function upload_sp_metadata_to_testshib {
+function upload_sp_metadata_to_samltest {
     local metadata_fname=${HOST_IP//./}_"$RANDOM"_sp
     local metadata_url=http://$HOST_IP/Shibboleth.sso/Metadata
 
@@ -84,39 +130,44 @@ function upload_sp_metadata_to_testshib {
         return
     fi
 
-    curl --form userfile=@"$FILES/${metadata_fname}" "https://www.testshib.org/procupload.php"
+    curl --form userfile=@"$FILES/${metadata_fname}" --form "submit=OK" "https://samltest.id/upload.php"
 }
 
 function configure_federation {
-    configure_apache
-
-    # Copy a templated /etc/shibboleth/shibboleth2.xml file...
-    sudo cp $FEDERATION_FILES/shibboleth2.xml $SHIBBOLETH_XML
-    # ... and replace the %HOST_IP%, %IDP_REMOTE_ID%,and %IDP_METADATA_URL% placeholders
-    sudo sed -i -e "
-        s|%HOST_IP%|$HOST_IP|g;
-        s|%IDP_REMOTE_ID%|$IDP_REMOTE_ID|g;
-        s|%IDP_METADATA_URL%|$IDP_METADATA_URL|g;
-        " $SHIBBOLETH_XML
-
-    sudo cp "$FEDERATION_FILES/attribute-map.xml" $ATTRIBUTE_MAP
-
-    restart_service shibd
-
-    # Enable the mapped auth method in /etc/keystone.conf
-    iniset $KEYSTONE_CONF auth methods "external,password,token,mapped"
-
     # Specify the header that contains information about the identity provider
     iniset $KEYSTONE_CONF mapped remote_id_attribute "Shib-Identity-Provider"
 
+    # Configure certificates and keys for Keystone as an IdP
+    if is_service_enabled tls-proxy; then
+        iniset $KEYSTONE_CONF saml certfile "$INT_CA_DIR/$DEVSTACK_CERT_NAME.crt"
+        iniset $KEYSTONE_CONF saml keyfile "$INT_CA_DIR/private/$DEVSTACK_CERT_NAME.key"
+    else
+        openssl genrsa -out /etc/keystone/ca.key 4096
+        openssl req -new -x509 -days 1826 -key /etc/keystone/ca.key -out /etc/keystone/ca.crt \
+            -subj "/C=US/ST=Denial/L=Springfield/O=Dis/CN=www.example.com"
+
+
+        iniset $KEYSTONE_CONF saml certfile "/etc/keystone/ca.crt"
+        iniset $KEYSTONE_CONF saml keyfile "/etc/keystone/ca.key"
+    fi
+
+    iniset $KEYSTONE_CONF saml idp_entity_id "$KEYSTONE_AUTH_URI/v3/OS-FEDERATION/saml2/idp"
+    iniset $KEYSTONE_CONF saml idp_sso_endpoint "$KEYSTONE_AUTH_URI/v3/OS-FEDERATION/saml2/sso"
+    iniset $KEYSTONE_CONF saml idp_metadata_path "/etc/keystone/keystone_idp_metadata.xml"
+
     if [[ "$WSGI_MODE" == "uwsgi" ]]; then
         restart_service "devstack@keystone"
     fi
 
+    keystone-manage saml_idp_metadata > /etc/keystone/keystone_idp_metadata.xml
+
+    configure_shibboleth
+    configure_apache
+
     # TODO(knikolla): We should not be relying on an external service. This
     # will be removed once we have an idp deployed during devstack install.
-    if [[ "$IDP_ID" == "testshib" ]]; then
-        upload_sp_metadata_to_testshib
+    if [[ "$IDP_ID" == "samltest" ]]; then
+        upload_sp_metadata_to_samltest
     fi
 }
 
@@ -131,8 +182,15 @@ function register_federation {
 }
 
 function configure_tests_settings {
+    # Enable the mapped auth method in /etc/keystone.conf
+    iniset $KEYSTONE_CONF auth methods "external,password,token,mapped"
+
     # Here we set any settings that might be need by the fed_scenario set of tests
     iniset $TEMPEST_CONFIG identity-feature-enabled federation True
+    # If not using samltest as an external IdP, tell tempest not to test that scenario
+    if [[ "$IDP_ID" != "samltest" ]] ; then
+        iniset $TEMPEST_CONFIG identity-feature-enabled external_idp false
+    fi
 
     # Identity provider settings
     iniset $TEMPEST_CONFIG fed_scenario idp_id $IDP_ID
@@ -146,6 +204,7 @@ function configure_tests_settings {
     iniset $TEMPEST_CONFIG fed_scenario mapping_user_name $MAPPING_USER_NAME
     iniset $TEMPEST_CONFIG fed_scenario mapping_group_name $GROUP_NAME
     iniset $TEMPEST_CONFIG fed_scenario mapping_group_domain_name $DOMAIN_NAME
+    iniset $TEMPEST_CONFIG fed_scenario enable_k2k_groups_mapping True
 
     # Protocol settings
     iniset $TEMPEST_CONFIG fed_scenario protocol_id $PROTOCOL_ID
@@ -154,7 +213,14 @@ function configure_tests_settings {
 function uninstall_federation {
     if is_ubuntu; then
         uninstall_package libapache2-mod-shib2
+    elif is_fedora; then
+        uninstall_package shibboleth
+
+        # Remove Shibboleth repository
+        sudo rm /etc/yum.repos.d/shibboleth.repo
+    elif is_suse; then
+        unistall_package shibboleth-sp
     else
-        echo "Skipping uninstallation of shibboleth for non ubuntu host"
+        echo "Skipping uninstallation of shibboleth for non ubuntu nor fedora nor suse host"
     fi
 }
